/*
 * Copyright (c) 2019-2020 Nordic Semiconductor ASA
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#if CONFIG_BOARD_CUSTOM_NRF5340_CPUAPP

#include <zephyr/drivers/gpio/gpio_utils.h>
#include <zephyr/drivers/gpio.h>
#include <zephyr/drivers/i2c.h>
#include <zephyr/logging/log.h>
#include <zephyr/sys/byteorder.h>
#include <zephyr/sys/util.h>
#include "error.h"

/* TCA6424A auto increment register addresses */
#define TCA6424A_REG_ADDR_INPUT(port)              (0x80 + port)
#define TCA6424A_REG_ADDR_OUTPUT(port)             (0x84 + port)
#define TCA6424A_REG_ADDR_POLARITY_INVERSION(port) (0x88 + port)
#define TCA6424A_REG_ADDR_CONFIGURATION(port)      (0x8C + port)
#define I2C_INST DT_NODELABEL(i2c0)
#define TCA6424A_ADDRESS  0x22 

const struct device *const _gI2cDev = DEVICE_DT_GET(I2C_INST);

LOG_MODULE_REGISTER(tca6424a);

/*
static int _tca6424aReadReg(uint8_t regAddr, uint8_t *buf) {
    printk("tca6424aReadReg(0x%x, %d)\n", regAddr, 1);
    const int ret = i2c_burst_read(_gI2cDev, TCA6424A_ADDRESS, regAddr, buf, 1);
    if (ret < 0) {
        printk("i2c_burst_read(0x%x) failed, ret=%d\n", regAddr, ret);
        return ERROR_I2C_READ_FAIL;
    }

    return 0;
}
*/

static int _tca6424aWriteReg(uint8_t regAddr, uint8_t regVal) {
    printk("tca6424aWriteReg(0x%x, 0x%x)\n", regAddr, regVal);
    const uint8_t buf[1] = { regVal };
    const int ret =  i2c_burst_write(_gI2cDev, TCA6424A_ADDRESS, regAddr, buf, 1);
    if (ret < 0) {
        printk("i2c_burst_write(0x%x) failed, ret=%d\n", regAddr, ret);
        return ERROR_I2C_WRITE_FAIL;
    }

    
    return 0;
}

static int _tca6424aWriteConfigReg(const int port, uint8_t regVal) {
    const uint8_t regAddr = TCA6424A_REG_ADDR_CONFIGURATION(port);
    const int ret         = _tca6424aWriteReg(regAddr, regVal);
    if (ret < 0) {
        return ERROR_I2C_WRITE_FAIL;
    }

   return 0;
}

static int _tca6424aWriteOutputReg(const int port, uint8_t regVal) {
    const uint8_t regAddr = TCA6424A_REG_ADDR_OUTPUT(port);
    const int ret         = _tca6424aWriteReg(regAddr, regVal);
    if (ret < 0) {
        return ERROR_I2C_WRITE_FAIL;
    }

   return 0;
}

int ioExpanderInit(void) {
    printk("ioExpanderInit\n");

    // configuration
    int ret = _tca6424aWriteConfigReg(0, 0x04); // pin 2 is as input
    if (ret < 0) {
        return ERROR_I2C_WRITE_FAIL;
    }

    ret = _tca6424aWriteConfigReg(1, 0x14); // pins 12 and 14 are as input
    if (ret < 0) {
        return ERROR_I2C_WRITE_FAIL;
    }

    ret = _tca6424aWriteConfigReg(2, 0x20); // pin 25 is as input
    if (ret < 0) {
        return ERROR_I2C_WRITE_FAIL;
    }

    // output
    ret = _tca6424aWriteOutputReg(0, 0xFB);
    if (ret < 0) {
        return ERROR_I2C_WRITE_FAIL;
    }

    ret = _tca6424aWriteOutputReg(1, 0xEB);
    if (ret < 0) {
        return ERROR_I2C_WRITE_FAIL;
    }

    ret = _tca6424aWriteOutputReg(2, 0xDF);
    if (ret < 0) {
        return ERROR_I2C_WRITE_FAIL;
    }

    return 0;
}

#endif